/******************************************************************************
* 
*  Copyright (C) 2012 - 2017 Texas Instruments Incorporated - http://www.ti.com/ 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
* 
*   Redistributions of source code must retain the above copyright 
*   notice, this list of conditions and the following disclaimer.
* 
*   Redistributions in binary form must reproduce the above copyright
*   notice, this list of conditions and the following disclaimer in the 
*   documentation and/or other materials provided with the   
*   distribution.
* 
*   Neither the name of Texas Instruments Incorporated nor the names of
*   its contributors may be used to endorse or promote products derived
*   from this software without specific prior written permission.
* 
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
* 
*  MSP432 SimpleLink C:\ti\simplelink_msp432p4_sdk_1_50_00_12
* 
*****************************************************************************/
/*******************************************************************************
 * MSP432 ADC14 - Single Channel Continuous Sample w/ Timer_A Trigger
 *
 * Description: In this ADC14 code example, a single input channel is sampled
 * using the standard reference specified in the reference definition and 
 * configuration variable. The ADC is setup to continuously sample/convert
 * from A10. The sample-and hold time and the source of the sample trigger is 
 * provided by Timer_A0.  Capture Compare Register 0 (CCR0) defines the sample
 * period while CCR1 defines the sample-hold time. The CCR0 value is 23 and 
 * CCR1 is set to 19, resulting in a sample period of 1us.
 * The ADC is setup to perform repeated single-channel measurements of channel 
 * A10. 
 * The DMA moves the results from the ADC to the resultsBuffer and
 * destination pointer is maintained in the DMA ISR.
 *
 * Timer_A0 is started once the GPIO interrupt for P1.4 is serviced. Upon 
 * completion the array resultsBuffer is transmitted via UART.
 *
 *                MSP-EXP432P401
 *             ------------------
 *         /|\|                  |
 *          | |                  |<--- P5.6 (Veref+)
 *          | |                  |<--- P5.7 (Veref-)
 *          | |                  |
 *          --|RST         P4.2  |<--- A11 (Analog Input)
 *            |            P4.3  |<--- A10 (Analog Input)
 *            |                  |
 *            |            P1.4  |<--- GPIO trigger to Start conversions
 *            |                  |
 *            |            P1.0  |---> Debug port to show GPIO ISR
 *            |            P1.2  |<--- UART Rx
 *            |            P1.3  |---> UART Tx
 *            |            P2.0  |---> Debug port to show DMA ISR
 *            |            P2.4  |---> Debug TA0.1, ADC trigger
 *            |                  |
 *
 * Author: Timothy Logan/ C. Sterzik
 ******************************************************************************/
/* DriverLib Includes */
#include <ti/devices/msp432p4xx/driverlib/driverlib.h>
#include <arm_math.h>

/* Standard Includes */
#include <stdint.h>
#include <stdbool.h>

#include <testConfig.h>

//#define DEBUG

//#define ARRAY_LENGTH                8192
#define ARRAY_LENGTH                16384
//#define NUMBER_OF_DMA_PING_PONGS    7 // ARRAY_LENGTH/1024  -1
#define NUMBER_OF_DMA_PING_PONGS    15 // ARRAY_LENGTH/1024  -1
#define REMAINDER                   ARRAY_LENGTH-(1024*NUMBER_OF_DMA_PING_PONGS)

#define ADC_REFERENCE_VALUE         2.5
//#define ADC_REFERENCE_VALUE         1.45
//#define ADC_REFERENCE_VALUE         1.2
//#define ADC_REFERENCE_VALUE         3.3

 const ADC_test_Config tstConfiguration =
{
//    .selectClkConfig = ADCOSC_SMCLK_MCLK_DCO,
//    .selectClkConfig = SMCLK_SMCLK_MCLK_DCO,
    .selectClkConfig = SMCLK_SMCLK_MCLK_HFXT,
//    .selectRefConfig = VREF_AVCC
    .selectRefConfig = VREF_EXT
//    .selectRefConfig = VREF_INT_2_5
//    .selectRefConfig = VREF_INT_1_45
//    .selectRefConfig = VREF_INT_1_2
};

 /*
  * UART Config
  *
  * SMCLK = 24Mhz
  * Baud = 9600
  */
 const eUSCI_UART_Config uartConfig =
 {
         EUSCI_A_UART_CLOCKSOURCE_SMCLK,          // SMCLK Clock Source
         156,                                       // BRDIV = 6
         4,                                       // UCxBRF = 8
         0,                                       // UCxBRS = 0x20
         EUSCI_A_UART_NO_PARITY,                  // No Parity
         EUSCI_A_UART_LSB_FIRST,                  // LSB First
         EUSCI_A_UART_ONE_STOP_BIT,               // One stop bit
         EUSCI_A_UART_MODE,                       // UART mode
         EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION  // Oversampling
 };

/*
 * Timer_A Compare Configuration Parameter
 * CCR1 is used to trigger the ADC14, conversion time
 * is defined by the resolution
 * 14bit -> 16 cycles + 1 cycle (SLAU356d, 20.2.8.3)
 *
 *  In this example, 14-bit resolution at 24Mhz ~708ns conversion time
 *  Sample time is defined by 5 ADC clocks, pulse sample mode
 *  Sample period is 24/24Mhz = 1us
 */
const Timer_A_PWMConfig timerA_PWM =
{
    .clockSource = TIMER_A_CLOCKSOURCE_SMCLK,
    .clockSourceDivider = TIMER_A_CLOCKSOURCE_DIVIDER_1,
    .timerPeriod = 23,
    .compareRegister = TIMER_A_CAPTURECOMPARE_REGISTER_1,
    .compareOutputMode = TIMER_A_OUTPUTMODE_SET_RESET,
    .dutyCycle = 19
};

/* DMA Control Table */
#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN(MSP_EXP432P401RLP_DMAControlTable, 1024)
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment=1024
#elif defined(__GNUC__)
__attribute__ ((aligned (1024)))
#elif defined(__CC_ARM)
__align(1024)
#endif
static DMA_ControlTable MSP_EXP432P401RLP_DMAControlTable[16];

void configure_UART(void);

/* Statics */
q15_t resultsBuffer[ARRAY_LENGTH];

uint8_t pingPongCounter;

uint16_t arrayOffset;
uint32_t arraySize, arrayMode;



int main(void)
{
    bool hfxtResult;

    /* Halting WDT  */
    MAP_WDT_A_holdTimer();
    MAP_Interrupt_enableSleepOnIsrExit();

    /* Terminating all remaining pins to minimize power consumption. This is
        done by register accesses for simplicity and to minimize branching API
        calls */
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P1, PIN_ALL8);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P2, PIN_ALL8);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_PB, PIN_ALL16);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_PC, PIN_ALL16);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_PD, PIN_ALL16);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_PE, PIN_ALL16);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_PJ, GPIO_PIN0 | GPIO_PIN1);

    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P1, PIN_ALL8);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P2, PIN_ALL8);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_PB, PIN_ALL16);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_PC, PIN_ALL16);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_PD, PIN_ALL16);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_PE, PIN_ALL16);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_PJ, GPIO_PIN0 | GPIO_PIN1);

    /* Configuring pins for peripheral/crystal usage and LED for output */
    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_PJ,
            GPIO_PIN3 | GPIO_PIN2, GPIO_PRIMARY_MODULE_FUNCTION);

    /*
     * Revision C silicon supports wait states of 1 at 48Mhz
     */
    MAP_PCM_setCoreVoltageLevel(PCM_VCORE1);
    MAP_PCM_setPowerMode(PCM_DCDC_MODE);
    MAP_FlashCtl_setWaitState(FLASH_BANK0, 1);
    MAP_FlashCtl_setWaitState(FLASH_BANK1, 1);

    /*
     * Setting up clocks
     * MCLK = MCLK = 48MHz
     * SMCLK = MCLK/2 = 24Mhz
     * ACLK = REFO = 32Khz
     *
     */
    if(tstConfiguration.selectClkConfig == SMCLK_SMCLK_MCLK_HFXT)
    {
        CS_setExternalClockSourceFrequency(32000,48000000);
        hfxtResult = CS_startHFXTWithTimeout(false,100000);
        if(hfxtResult == true)
        {
            /* Initializing MCLK to HFXT (effectively 48MHz) */
            MAP_CS_initClockSignal(CS_ACLK, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1);
            MAP_CS_initClockSignal(CS_SMCLK, CS_HFXTCLK_SELECT, CS_CLOCK_DIVIDER_2);
            MAP_CS_initClockSignal(CS_MCLK, CS_HFXTCLK_SELECT, CS_CLOCK_DIVIDER_1);
            //![Simple CS Config]
        }
        else
        {
            while(1); // HFXT failed
        }
    }
    else
    {
        MAP_CS_setDCOFrequency(48000000);
        /* Initializing MCLK to HFXT (effectively 48MHz) */
        MAP_CS_initClockSignal(CS_ACLK, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1);
        MAP_CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_2);
        MAP_CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
        //![Simple CS Config]
    }

#ifdef DEBUG
    /*
     * Debug
     * Configuring P1.0, P2.0, and P2.1 as outputs
     */
    P1OUT &= ~BIT0;
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0);
    P2OUT &= ~(BIT0+BIT1);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0 | GPIO_PIN1);
#endif

    /*
     * Configuring GPIOs (P4.2 A11)
     * P4.3 is A10, A10+
     * P4.2 is A11, A10-
     *
     * Dedicated Pins
     * P4.6 is A7 A6-
     * P4.7 is A6 A6+
     *
     */

    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P4, GPIO_PIN2 | GPIO_PIN3,
    GPIO_TERTIARY_MODULE_FUNCTION);

//    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P4, GPIO_PIN6 | GPIO_PIN7,
//    GPIO_TERTIARY_MODULE_FUNCTION);

    /*
     * Debug: set TA0.1 as output to see ADC trigger signal
     */
#ifdef DEBUG
    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P2, GPIO_PIN4,
    GPIO_PRIMARY_MODULE_FUNCTION);
#endif

    /*
     * Configuring P1.4 as an input and enabling interrupt, the timer is started from
     * GPIO ISR.
     */
    MAP_GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN4);
    MAP_GPIO_interruptEdgeSelect(GPIO_PORT_P1,GPIO_PIN4,GPIO_HIGH_TO_LOW_TRANSITION);
    MAP_GPIO_clearInterruptFlag(GPIO_PORT_P1, GPIO_PIN4);
    MAP_GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN4);

    configure_UART();

    if(tstConfiguration.selectRefConfig > VREF_EXT)
    {
        if(tstConfiguration.selectRefConfig == VREF_INT_1_2)
        {
            MAP_REF_A_setReferenceVoltage(REF_A_VREF1_2V);
        }
        if(tstConfiguration.selectRefConfig == VREF_INT_1_45)
        {
            MAP_REF_A_setReferenceVoltage(REF_A_VREF1_45V);
        }
        if(tstConfiguration.selectRefConfig == VREF_INT_2_5)
        {
            MAP_REF_A_setReferenceVoltage(REF_A_VREF2_5V);
        }
        MAP_REF_A_enableReferenceVoltage();
    }

    /*
     * Initializing ADC
     * Clock source either 25Mhz ADCOSC or 24Mhs SMCLK (sources from either DCO or HFXT)
     * 14-bit mode by default
     * Conversion mode is differential signed binary and treated as q15_t.
     * Result is placed into memory location ADC14MEM0
     * Reference selectable from Internal (AVcc, Vref_A) or external
     */
    MAP_ADC14_enableModule();
    if(tstConfiguration.selectClkConfig == ADCOSC_SMCLK_MCLK_DCO)
    {
        MAP_ADC14_initModule(ADC_CLOCKSOURCE_ADCOSC, ADC_PREDIVIDER_1, ADC_DIVIDER_1,
                0);
    }
    else
    {
        MAP_ADC14_initModule(ADC_CLOCKSOURCE_SMCLK, ADC_PREDIVIDER_1, ADC_DIVIDER_1,
                0);
    }

    /*
     * Configuring ADC Memory, repeat-single-channel, A0
     */
    MAP_ADC14_configureSingleSampleMode(ADC_MEM0, true);

    /*
     * Configuring ADC Memory, reference, and differential conversion
     * A10 goes to mem0
     */
    if(tstConfiguration.selectRefConfig == VREF_EXT)
    {
        /*
         * Configure GPIO. P5.6 (Veref+) and P5.7 (Veref-)
         */
        MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P5, GPIO_PIN6 | GPIO_PIN7,
        GPIO_TERTIARY_MODULE_FUNCTION);

        MAP_ADC14_configureConversionMemory(ADC_MEM0,ADC_VREFPOS_EXTPOS_VREFNEG_EXTNEG,
                                            ADC_INPUT_A10, ADC_DIFFERENTIAL_INPUTS);
//        MAP_ADC14_configureConversionMemory(ADC_MEM0,ADC_VREFPOS_EXTPOS_VREFNEG_EXTNEG,
//                                            ADC_INPUT_A6, ADC_DIFFERENTIAL_INPUTS);
    }
    else if(tstConfiguration.selectRefConfig == VREF_AVCC)
    {
        MAP_ADC14_configureConversionMemory(ADC_MEM0,ADC_VREFPOS_AVCC_VREFNEG_VSS,
                                            ADC_INPUT_A10, ADC_DIFFERENTIAL_INPUTS);

//        MAP_ADC14_configureConversionMemory(ADC_MEM0,ADC_VREFPOS_AVCC_VREFNEG_VSS,
//                                            ADC_INPUT_A6, ADC_DIFFERENTIAL_INPUTS);
    }
    else
    {
        MAP_ADC14_configureConversionMemory(ADC_MEM0,ADC_VREFPOS_INTBUF_VREFNEG_VSS,
                                            ADC_INPUT_A10, ADC_DIFFERENTIAL_INPUTS);

//        MAP_ADC14_configureConversionMemory(ADC_MEM0,ADC_VREFPOS_INTBUF_VREFNEG_VSS,
//                                            ADC_INPUT_A6, ADC_DIFFERENTIAL_INPUTS);
    }

    MAP_ADC14_setResultFormat(ADC_SIGNED_BINARY);

    /*
     * Configuring the sample trigger to be sourced from Timer_A0 CCR1 and on the
     * rising edge, default samplemode is extended (SHP=0)
     */
    MAP_ADC14_setSampleHoldTrigger(ADC_TRIGGER_SOURCE1, false);

    /* Enabling conversions */
    MAP_ADC14_enableConversion();

    /* Configuring DMA module */
    MAP_DMA_enableModule();
    MAP_DMA_setControlBase(MSP_EXP432P401RLP_DMAControlTable);

    /*
     * Setup the DMA + ADC14 interface
     */
    MAP_DMA_disableChannelAttribute(DMA_CH7_ADC14,
                                 UDMA_ATTR_ALTSELECT | UDMA_ATTR_USEBURST |
                                 UDMA_ATTR_HIGH_PRIORITY |
                                 UDMA_ATTR_REQMASK);

    /*
     * Setting Control Indexes. In this case we will set the source of the
     * DMA transfer to ADC14 Memory 0 and the destination to the destination
     * data array.
     */
    MAP_DMA_setChannelControl(UDMA_PRI_SELECT | DMA_CH7_ADC14,
            UDMA_SIZE_16 | UDMA_SRC_INC_NONE | UDMA_DST_INC_16 | UDMA_ARB_1);
    MAP_DMA_setChannelTransfer(UDMA_PRI_SELECT | DMA_CH7_ADC14,
            UDMA_MODE_PINGPONG, (void*) &ADC14->MEM[0],
            (void*)resultsBuffer, 1024);
    MAP_DMA_setChannelControl(UDMA_ALT_SELECT | DMA_CH7_ADC14,
            UDMA_SIZE_16 | UDMA_SRC_INC_NONE | UDMA_DST_INC_16 | UDMA_ARB_1);
    MAP_DMA_setChannelTransfer(UDMA_ALT_SELECT | DMA_CH7_ADC14,
            UDMA_MODE_PINGPONG, (void*) &ADC14->MEM[0],
            (void*)&resultsBuffer[1024], 1024);

    /* Assigning/Enabling Interrupts */
    MAP_DMA_assignInterrupt(DMA_INT1, 7);
    MAP_DMA_assignChannel(DMA_CH7_ADC14);
    MAP_DMA_clearInterruptFlag(7);

    /* Enabling Interrupts */
    MAP_Interrupt_enableInterrupt(INT_DMA_INT1);
    MAP_Interrupt_enableInterrupt(INT_PORT1);
    MAP_Interrupt_enableMaster();

    /* Going to sleep */
    MAP_PCM_gotoLPM0();
    __no_operation();
}

/*
 *  15 Primary completed   resultArray[0-1023]
 *  14 Alternate completed resultArray[1024-2047]
 *
 *  3  Primary completed   resultArray[12288-13311]
 *  2  Alternate Completed resultArray[13312-14335] * setup last alternate here
 *  1  Primary completed   resultArray[14336-15359]
 *  0  Alternate completed resultArray[15360-15999]
 *
 */
__attribute__((ramfunc))  // Requires compiler TI v15.12.1.LTS
void DMA_INT1_IRQHandler(void)
{
    uint8_t * uartPtr;
    uint32_t ii;

#ifdef DEBUG
    BITBAND_PERI(P2->OUT, 0) = 1;
#endif

    if(pingPongCounter == 0)
    {
        MAP_Timer_A_stopTimer(TIMER_A0_BASE);
        MAP_DMA_disableChannel(7);
        MAP_DMA_disableChannelAttribute(DMA_CH7_ADC14,UDMA_ATTR_ALTSELECT);

//        MAP_UART_transmitData(EUSCI_A0_BASE,tstConfiguration.selectClkConfig);
//        MAP_UART_transmitData(EUSCI_A0_BASE,tstConfiguration.selectRefConfig);

        for(ii=0;ii<ARRAY_LENGTH;ii++)
        {
            /*
             * LSB first
             */
            uartPtr = (uint8_t*)&resultsBuffer[ii];
            MAP_UART_transmitData(EUSCI_A0_BASE,*uartPtr++);
            MAP_UART_transmitData(EUSCI_A0_BASE,*uartPtr);
            __delay_cycles(150);
        }

        /*
         * Setting Control Indexes. In this case we will set the source of the
         * DMA transfer to ADC14 Memory 0 and the destination to the destination
         * data array.
         */
        MAP_DMA_setChannelControl(UDMA_PRI_SELECT | DMA_CH7_ADC14,
                UDMA_SIZE_16 | UDMA_SRC_INC_NONE | UDMA_DST_INC_16 | UDMA_ARB_1);
        MAP_DMA_setChannelTransfer(UDMA_PRI_SELECT | DMA_CH7_ADC14,
                UDMA_MODE_PINGPONG, (void*) &ADC14->MEM[0],
                (void*)resultsBuffer, 1024);
        MAP_DMA_setChannelControl(UDMA_ALT_SELECT | DMA_CH7_ADC14,
                UDMA_SIZE_16 | UDMA_SRC_INC_NONE | UDMA_DST_INC_16 | UDMA_ARB_1);
        MAP_DMA_setChannelTransfer(UDMA_ALT_SELECT | DMA_CH7_ADC14,
                UDMA_MODE_PINGPONG, (void*) &ADC14->MEM[0],
                (void*)&resultsBuffer[1024], 1024);
    }
    else
    {
        if(pingPongCounter > 1)
        {
            arrayOffset = 1024*(NUMBER_OF_DMA_PING_PONGS-pingPongCounter+2);
            if(pingPongCounter == 2)
            {
                arraySize = REMAINDER;
                arrayMode = UDMA_MODE_BASIC;
            }
            else
            {
                arraySize = 1024;
                arrayMode = UDMA_MODE_PINGPONG;
            }
            /*
             * Switch between primary and alternate bufferes with DMA's PingPong mode
             */
            if (MAP_DMA_getChannelAttribute(7) & UDMA_ATTR_ALTSELECT)
            {
//                MAP_DMA_setChannelTransfer(UDMA_PRI_SELECT | DMA_CH7_ADC14, arrayMode,
//                                           (void*) &ADC14->MEM[0],
//                                           (void*)&resultsBuffer[arrayOffset],arraySize);

                MSP_EXP432P401RLP_DMAControlTable[7].dstEndAddr =
                        (void*)&resultsBuffer[arrayOffset+arraySize];
                MSP_EXP432P401RLP_DMAControlTable[7].control =
                        (MSP_EXP432P401RLP_DMAControlTable[7].control & 0xff000000 ) |
                        (((arraySize)-1)<<4) | arrayMode;
            }
            else
            {
//                MAP_DMA_setChannelTransfer(UDMA_ALT_SELECT | DMA_CH7_ADC14, arrayMode,
//                                           (void*)&ADC14->MEM[0],
//                                           (void*)&resultsBuffer[arrayOffset],arraySize);
                MSP_EXP432P401RLP_DMAControlTable[15].dstEndAddr =
                        (void*)&resultsBuffer[arrayOffset+arraySize];
                MSP_EXP432P401RLP_DMAControlTable[15].control =
                        (MSP_EXP432P401RLP_DMAControlTable[15].control & 0xff000000 ) |
                        (((arraySize)-1)<<4) | arrayMode;
            }
        }
        pingPongCounter--;
    }

#ifdef DEBUG
    BITBAND_PERI(P2->OUT, 0) = 0;
#endif
}

void configure_UART(void)
{
    /* Configuring UART Module */
    MAP_UART_initModule(EUSCI_A0_BASE, &uartConfig);

    /* Enable UART module */
    MAP_UART_enableModule(EUSCI_A0_BASE);

    /* Selecting P1.2 and P1.3 in UART mode */
    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1,
            GPIO_PIN2 | GPIO_PIN3, GPIO_PRIMARY_MODULE_FUNCTION);
}

void PORT1_IRQHandler(void)
{
#ifdef DEBUG
    BITBAND_PERI(P1->OUT, 0) = 1;
#endif
    P1IFG &= ~BIT4;
    if(pingPongCounter==0)
    {
        pingPongCounter  = NUMBER_OF_DMA_PING_PONGS;
        MAP_DMA_enableChannel(7);
        MAP_Timer_A_generatePWM(TIMER_A0_BASE, &timerA_PWM);
    }
#ifdef DEBUG
    BITBAND_PERI(P1->OUT, 0) = 0;
#endif
}
